<?php


//!	RFC3986 形式で URL エンコードする関数
//!	
//!	@param $str 変換前URL文字列
//!	@return sring 変換後URL文字列
function urlencode_rfc3986($str)
{
    return str_replace('%7E', '~', rawurlencode($str));
}

//!	URL(文字列)を編集する関数
//!	
//!	@param $params WEB APIに渡すパラメータリスト
//!	@param $base_param WEB APIに渡す基本パラメータ
//!	@param $baseurl WEB APIの基本URL
//!	@param $id カテゴリID
//!
//!	@return sring 編集後URL文字列
function getURL(&$params,$baseurl,$id = -1)
{
	if(isset($id) && $id>=0){
		$params['BrowseNodeId']    =$id;   // category (pc/software)
	}

	// パラメータの順序を昇順に並び替えます
	ksort($params);

	// canonical string を作成します
	$canonical_string = '';
	foreach ($params as $k => $v) {
	    $canonical_string .= '&'.urlencode_rfc3986($k).'='.urlencode_rfc3986($v);
	}
	$canonical_string = substr($canonical_string, 1);
	
	// 署名を作成します
	// - 規定の文字列フォーマットを作成
	// - HMAC-SHA256 を計算
	// - BASE64 エンコード
	$parsed_url = parse_url($baseurl);
	$string_to_sign = "GET\n{$parsed_url['host']}\n{$parsed_url['path']}\n{$canonical_string}";
	
	//	証明書をエンコードする
	$signature = base64_encode(hash_hmac('sha256', $string_to_sign, '**********************', true));	//	ここで秘密キーを設定して暗号化する
	
	// URL を作成します
	// - リクエストの末尾に署名を追加
	return  $baseurl.'?'.$canonical_string.'&Signature='.urlencode_rfc3986($signature);

}
//!	親カテゴリを探索・編集する関数
//!	
//!	@param $current WEB APIで取得した親カテゴリ情報
//!	@param $URI 階層ページへのリンク用基本アドレス
//!	@param $re_count 現在の階層番号 - 入出力情報
//!
//!	@return sring 親カテゴリ情報 HTML
function getParent(&$current,$URI,&$re_count)
{
	$cat_html = '';
	if(isset($current)){
		if(isset($current->Ancestors) && isset($current->Ancestors->BrowseNode)){
			$cat_html.= getParent($current->Ancestors->BrowseNode,$URI,$re_count);
		}
		$cat_html .= '<ul>';
		$cat_html .= '<li>階層 - '.($re_count+1).' : <a href="'.$URI.'?id='.$current->BrowseNodeId.'">'.$current->Name.' : '.$current->BrowseNodeId.'</a></li>';
	//	$cat_html .= '</ul>';
		$re_count++;
	}
	return $cat_html;
}
//!	初期画面を編集する関数
//!	
//!	@param $URI 各ページへのリンク用基本アドレス
//!
//!	@return sring 初期画面情報 HTML
function getTopCategory($URI)
{
	$cat_html = '';

	$cat_html .= "\n<h4>本・漫画・雑誌</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=465392">本</a> : 465392</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=52033011">洋書</a> : 52033011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=466280">漫画</a> : 466280</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13384021">雑誌</a> : 13384021</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=255460011">古本</a> : 255460011</li>';
	
	$cat_html .= "</ul>\n<h4>DVD・ミュージック・ゲーム</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=561958">DVD</a> : 561958</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=403507011">ブルーレイ</a> : 403507011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=561956">ミュージック</a> : 561956</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=637394">TVゲーム</a> : 637394</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=689132">PCゲーム</a> : 689132</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=2123629051">楽器</a> : 2123629051</li>';
	
	$cat_html .= "</ul>\n<h4>家電＆カメラ</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=16462091">カメラ・デジタルカメラ</a> : 16462091</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=3477381">テレビ・レコーダー</a> : 3477381</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=16462081">オーディオ</a> : 16462081</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=124048011">生活家電</a> : 124048011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=3895751">理美容家電</a> : 3895751</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=699913011">携帯電話</a> : 699913011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=3210981">すべての家電＆カメラ</a> : 3210981</li>';
	$cat_html .= "</ul>\n<h4>パソコン・オフィス用品</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=3371341">パソコン・周辺機器</a> : 3371341</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=637392">PCソフト</a> : 637392</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=689132">PCゲーム</a> : 689132</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=86731051">文房具・オフィス用品</a> : 86731051</li>';
	
	$cat_html .= "</ul>\n<h4>ホーム＆キッチン</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=13938481">キッチン＆テーブルウェア</a> : 13938481</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13938531">インテリア・収納・寝具</a> : 13938531</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13938541">生活雑貨・ペット用品</a> : 13938541</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=124048011">生活家電</a> : 124048011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=3828871">すべてのホーム＆キッチン</a> : 3828871</li>';
	
	$cat_html .= "</ul>\n<h4>食品＆飲料</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=70903051">食品</a> : 70903051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=71314051">スイーツ・お菓子</a> : 71314051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=71442051">ドリンク</a> : 71442051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=71588051">お酒</a> : 71588051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=57239051">すべての食品＆飲料</a> : 57239051</li>';
	$cat_html .= "</ul>\n<h4>ヘルス＆ビューティー</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=52374051">コスメ</a> : 52374051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=170039011">フェイス＆ボディケア</a> : 170039011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=169911011">ヘルスケア</a> : 169911011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=344024011">サプリメント</a> : 344024011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=14315501">フィットネス・トレーニング</a> : 14315501</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=160384011">すべてのヘルス＆ビューティー</a> : 160384011</li>';
	
	$cat_html .= "</ul>\n<h4>ベビー・おもちゃ・ホビー</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=344845011">ベビー＆マタニティ</a> : 344845011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13299531">おもちゃ</a> : 13299531</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13321861">ホビー</a> : 13321861</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=2123629051">楽器</a> : 2123629051</li>';
	$cat_html .= "</ul>\n<h4>ファッション・時計</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=352484011">服＆ファッション小物</a> : 352484011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=2016926051">シューズ＆バッグ</a> : 2016926051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=85895051">ジュエリー</a> : 85895051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=324025011">時計</a> : 324025011</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=2113178051">すべてのファッション＆時計</a> : 2113178051</li>';
	$cat_html .= "</ul>\n<h4>スポーツ＆アウトドア</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=15337751">自転車</a> : 15337751</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=14315411">アウトドア</a> : 14315411</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=14315501">フィットネス・トレーニング</a> : 14315501</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=14315441">ゴルフ</a> : 14315441</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=14304371">すべてのスポーツ＆アウトドア</a> : 14304371</li>';
	
	$cat_html .= "</ul>\n<h4>DIY・工具・車用品</h4>\n<ul>";
	$cat_html .= '<li><a href="'.$URI.'?id=2016929051">DIY・工具</a> : 2016929051</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=13945231">ガーデン</a> : 13945231</li>';
	$cat_html .= '<li><a href="'.$URI.'?id=2017304051">カー・バイク用品</a> : 2017304051</li>';

	return $cat_html;
}



//	-----------------------------
//	main 
//	-----------------------------
//	パラメータをチェック

if(isset($_REQUEST['id'])){
	$id = trim($_REQUEST['id']);
	if($id==''){
		$id = -1;
	} else {
		$id =(int)$id;
	}
} else {
	$id = -1;
}
$URI = $_SERVER['REQUEST_URI'];
if(stripos($URI,'?')!==false){
	$URI = substr($URI,0,stripos($URI,'?'));
}

$cat_html = '';

if($id<0){
	$cat_html = getTopCategory($URI);

} else {

	$baseurl = 'http://ecs.amazonaws.jp/onca/xml';
	
	$params = array();
	
	$params['Service']        = 'AWSECommerceService';	//	サービス名
	$params['AWSAccessKeyId'] = '**********************';	//	あなたの WEB API ID を設定します。
	$params['AssociateTag']   = '**********************';    // ← アソシエイトID
	$params['ResponseGroup']  = 'BrowseNodeInfo,TopSellers';    // 取得する情報グループ名 ここでは、属性情報、価格情報、イメージ情報、ユーザコメント情報
	$params['Version']        = '2010-06-01';          //	バージョン情報
	$params['Operation']      = 'BrowseNodeLookup';    // ← ItemSearch オペレーションの例

	$params['BrowseNodeId']      = $id;    //カテゴリーID
	
	// Timestamp パラメータを追加します
	// - 時間の表記は ISO8601 形式、タイムゾーンは UTC(GMT)
	$params['Timestamp'] = gmdate('Y-m-d\TH:i:s\Z');
	

	// create URL strings.
	$url = getURL($params,$baseurl);	//	service = itemSearch 
	
	// request to amazon !!
	$response = file_get_contents($url);
	
	// response to strings.
	$parsed_xml = null;
	if(isset($response)){
		$parsed_xml = simplexml_load_string($response);
	}
	
	$cat_html = '';
	if(
	    $response==false || !isset($parsed_xml) || 
	   (!isset($parsed_xml->BrowseNodes) || !isset($parsed_xml->BrowseNodes->BrowseNode)) || 
	   (isset($parsed_xml->BrowseNodes->Request->Errors) && !empty($parsed_xml->BrowseNodes->Request->Errors))
	   ){
		//	error
		$cat_html .= "******Error(Amazon)*******  <br />\n";
		$cat_html .= " 指定されたID = $id は、存在しないカテゴリのようです。<br />\n<br />\n";
		$cat_html .= getTopCategory($URI);
	} else {
		if(isset($parsed_xml->BrowseNodes)){
			$current = $parsed_xml->BrowseNodes->BrowseNode;
			if(isset($current)){
				$cat_html .= '<br />';
				$cat_html .= '<a href="'.$URI.'?id=">TOPのカテゴリ一覧へ</a><br />';
				$cat_html .= '<br />';
				$cat_html .= '現在のカテゴリ情報：<br />';
				$cat_html .= '<a href="'.$URI.'?id='.$current->BrowseNodeId.'">'.$current->Name.'</a> : '.$current->BrowseNodeId.'<br />';
	
				$cat_html .= '<br />';
				$cat_html .= 'カテゴリツリー：<br />';
				
				//	親カテゴリ
				$re_count  = 0;
				$cat_html .= '<ul>';
				$cat_html .= '<li>階層 - 0 : <a href="'.$URI.'?id=">root</a> : 0</li>';
				$re_count++;
				
				if(isset($current->Ancestors) && isset($current->Ancestors->BrowseNode)){
					$cat_html.= getParent($current->Ancestors->BrowseNode,$URI,$re_count);
				}
				$cat_html .= '<ul>';
				$cat_html .= '<li>階層 - '.($re_count+1).' : <a href="'.$URI.'?id='.$current->BrowseNodeId.'">'.$current->Name.'</a> : '.$current->BrowseNodeId.'</li>';
				$re_count++;

				//	子カテゴリ
				if(isset($current->Children)){
					$cat_html .= '<ul>';
					foreach($current->Children->BrowseNode as $cate_child){
						$cat_html .= '<li><a href="'.$URI.'?id='.$cate_child->BrowseNodeId.'">'.$cate_child->Name.'</a> : '.$cate_child->BrowseNodeId.'</li>';
					}
					$cat_html .= '</ul>';
				}
				for($ni=0;$ni<$re_count;$ni++){
					$cat_html .= '</ul>';
				}

			}
		} else {
			$cat_html = '**** DEBUG ***<br />';
			$cat_html .= print_r($parsed_xml,true);
		}
	}
}


?>

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="ja">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf8"> 
  <title>Amazon カテゴリID</title> 
</head>
<body>

<form action="<?php echo $URI; ?>" name="searchCategoryID" method="post">
<b>ID検索 ：</b><input type="text" name="id" value="<?php echo ($id<0? '':$id); ?>" /><input type="submit" value="検索" />
</form>
<br clear="left" />
<?php
	echo $cat_html;
?>
</body>
</html>
